@@ -1,19 +1,21 @@
-"""Checker for global variables
-"""
+"""Checker for global variables"""
 
 from __future__ import annotations
 
 import re
+from typing import TYPE_CHECKING
 
 from astroid import nodes
 from pylint.checkers import BaseChecker
 from pylint.checkers.base import UpperCaseStyle
 from pylint.checkers.base.name_checker.checker import DEFAULT_PATTERNS
-from pylint.checkers.utils import is_builtin
-from pylint.lint import PyLinter
+from pylint.checkers.utils import is_builtin, only_required_for_messages
 
 from python_ta.utils import _is_in_main
 
+if TYPE_CHECKING:
+    from pylint.lint import PyLinter
+
 
 class GlobalVariablesChecker(BaseChecker):
     """A checker class that reports the forbidden global variables in the file"""
@@ -31,26 +33,31 @@ class GlobalVariablesChecker(BaseChecker):
         super().__init__(linter)
         self.import_names = []
 
+    @only_required_for_messages("forbidden-global-variables")
     def visit_global(self, node: nodes.Global) -> None:
         args = "the keyword 'global' is used on line {}".format(node.lineno)
         self.add_message("forbidden-global-variables", node=node, args=args)
 
+    @only_required_for_messages("forbidden-global-variables")
     def visit_assignname(self, node: nodes.AssignName) -> None:
         """Allow global constant variables (uppercase) and type aliases (type alias pattern), but issue messages for
         all other globals.
         """
         self._inspect_vars(node)
 
+    @only_required_for_messages("forbidden-global-variables")
     def visit_name(self, node: nodes.Name) -> None:
         """Allow global constant variables (uppercase) and type aliases (type alias pattern), but issue messages for
         all other globals.
         """
         self._inspect_vars(node)
 
+    @only_required_for_messages("forbidden-global-variables")
     def visit_import(self, node: nodes.Import) -> None:
         """Save the names of imports, to prevent mistaking for global vars."""
         self._store_name_or_alias(node)
 
+    @only_required_for_messages("forbidden-global-variables")
     def visit_importfrom(self, node: nodes.ImportFrom) -> None:
         """Save the names of imports, to prevent mistaking for global vars."""
         self._store_name_or_alias(node)
